import React, { useRef, useState } from 'react';
import { PageContainer } from '@ant-design/pro-layout';
import { Alert, Button, Card, message, Space, Modal, Table } from 'antd';
import { FileExcelOutlined, SaveOutlined, FileZipOutlined } from '@ant-design/icons';
import School from '@/components/School';
import Step from './Step'
import { read } from './excel'

export default (props) => {
  
  const [visible, setVisible] = useState(false)
  const [schoolId, setSchoolId] = useState()
  const [reading, setReading] = useState(false)

  const workbookRef = useRef()
  const [dataSource, setDataSource] = useState([])
  const [zipFile, setZipFile] = useState()

  const columns = [
    {
      key: 'index',
      title: '序号',
      width: 200,
      align: 'center',
      render: (t,r,i) => i + 1,
    },
    {
      key: 'testNo',
      dataIndex: 'testNo',
      title: '体检号',
      align: 'center',
    },
    {
      key: 'studentNo',
      dataIndex: 'studentNo',
      title: '学号',
      align: 'center',
    },
  ]

  const handleFile = () => {
    const input = document.createElement('input')
    input.type = 'file'
    input.accept = 'application/vnd.ms-excel,application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
    input.onchange = (e) => {
      const file = e.target.files[0]
      if (!file) {
        message.warning('请选择文件')
      }

      setReading(true)
      read(file).then((res) => {
        workbookRef.current = res
        setReading(false)
        setVisible(true)
      }).catch((err) => {
        console.error(err)
        setReading(false)
      })
    }
    input.click()
  }

  const handleZipFile = () => {
    const input = document.createElement('input')
    input.type = 'file'
    input.accept = 'application/zip'
    input.onchange = (e) => {
      const file = e.target.files[0]
      if (!file) {
        message.warning('请选择文件')
      }
      setZipFile(file)
    }
    input.click()

  }

  const handleSettingChange = (e) => {
    setVisible(false)

    const { dataStartRow, sheet, cols } = e

    const [, end] = sheet['!ref'].split(':')
    const maxRow = /\d+/.exec(end)[0] - 0
    const arr = []
    for (let i = dataStartRow; i <= maxRow; i += 1) {
      const testNo = sheet[`${cols.testNoCol}${i}`]?.v
      const studentNo = sheet[`${cols.studentNoCol}${i}`]?.v

      arr.push({
        testNo,
        studentNo, 
      })
    }

    setDataSource(arr)
  }

  return (
    <PageContainer>
      <Space direction="vertical" size="large" style={{width: '100%'}}>
        <Alert message="友情提示" description="导入文件数据不要超过 200 列, 6000 行。如果出现未知错误, 请退出重试几次" type="info" closable />
        <Card>
          <Space direction="horizontal" size="large">
            <School
              autoDefault={false}
              value={schoolId}
              onChange={setSchoolId}
              allowClear
              style={{width: '300px'}}
              placeholder="请选择学校"
            ></School>

            <Button
              ghost
              type="primary"
              loading={reading}
              disabled={!schoolId}
              icon={<FileExcelOutlined />}
              onClick={handleFile}
            >上传 Excel 文件</Button>

            <Button
              ghost
              type="primary"
              disabled={!dataSource ||!dataSource.length}
              icon={<FileZipOutlined />}
              onClick={handleZipFile}
            >上传 ZIP 文件</Button>
            
            <Button
              type="primary"
              loading={reading}
              disabled={!zipFile}
              icon={<SaveOutlined />}
              onClick={handleFile}
            >保存入库</Button>
          </Space>
        </Card>

        <Table columns={columns} dataSource={dataSource} rowKey="testNo" pagination={false} />
      </Space>
      <Modal
        title="设置数据内容"
        visible={visible}
        onCancel={() => setVisible(false)}
        footer={null}
        width={480}
        maskClosable={false}
      >
        <Step workbook={workbookRef.current} onChange={handleSettingChange} />
      </Modal>
    </PageContainer>
  )
}
